﻿//////////////////////////////////////////////////
//             CompositeBuilder.cs              //
//         Part of RebornCo by kaihaider        //
//////////////////////////////////////////////////
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using ff14bot.Enums;
using ff14bot.Helpers;
using RebornCo.Helpers;
using TreeSharp;

namespace RebornCo
{
    public static class CompositeBuilder
    {
        /// <summary>
        /// allows generic behaviors to query current type of behavior
        /// during behavior construction
        /// </summary>
        public static BehaviorType CurrentBehaviorType { get; set; }

        public static bool SilentBehaviorCreation { get; set; }


        public static List<MethodInfo> _methods = new List<MethodInfo>();

        public static Composite GetComposite(RebornCoRoutine whom,BehaviorType behavior, ClassJobType job, GameContext context, out int behaviourCount, bool silent = false)
        {
            if (context == GameContext.None)
            {
                // None is an invalid context, but rather than stopping bot wait it out with donothing logic
                //Logging.Write(Colors.White, "No Active Context -{0}{1} for{2} set to DoNothingBehavior temporarily", wowClass.ToString().CamelToSpaced(), behavior.ToString().CamelToSpaced(), spec.ToString().CamelToSpaced());
                behaviourCount = 1;
                return null; //NoContextAvailable.CreateDoNothingBehavior();
            }

            SilentBehaviorCreation = silent;
            behaviourCount = 0;
            /*
            if (_methods.Count <= 0)
            {
                // Logger.WriteDebug("Singular Behaviors: building method list");
                foreach (var type in Assembly.GetExecutingAssembly().GetTypes())
                {
                    // All behavior methods should not be generic, and should have zero parameters, with their return types being of type Composite.
                    _methods.AddRange(
                        type.GetMethods(BindingFlags.Static | BindingFlags.Public).Where(
                            mi => !mi.IsGenericMethod && mi.GetParameters().Length == 0).Where(
                                mi => mi.ReturnType.IsAssignableFrom(typeof(Composite))));
                }
                Logger.WriteFile("RebornCo Behaviors: Added " + _methods.Count + " behaviors");
            }  */

            //_methods.Clear();
            if (_methods.Count <= 0)
            {
                _methods.AddRange(whom.GetType().GetMethods(BindingFlags.Public | BindingFlags.Instance).Where(
                    mi => !mi.IsGenericMethod && mi.GetParameters().Length == 0).Where(
                        mi => mi.ReturnType.IsAssignableFrom(typeof (Composite))));

                Logging.Write("RebornCo Behaviors: Added " + _methods.Count + " behaviors");
            }

            var matchedMethods = new Dictionary<BehaviorAttribute, Composite>();

            foreach (MethodInfo mi in _methods)
            {
                // If the behavior is set as ignore. Don't use it? Duh?
                if (mi.GetCustomAttributes(typeof (IgnoreBehaviorCountAttribute), false).Any())
                    continue;

                // If there's no behavior attrib, then move along.
                foreach (var a in mi.GetCustomAttributes(typeof (BehaviorAttribute), false))
                {
                    var attribute = a as BehaviorAttribute;
                    if (attribute == null)
                        continue;


                    //Logging.Write("RebornCo: Finding Match");
                    // Check if our behavior matches with what we want. If not, don't add it!
                    if (IsMatchingMethod(attribute, behavior, context, job))
                    {
                        //if (!silent)
                        //Logging.Write("RebornCo: Found Match");
                        //Logging.Write(LogLevel.Diagnostic, "RebornCo: {0} {1} {2}", job.ToString(), behavior.ToString(), mi.Name);

                        CurrentBehaviorType = behavior;

                        // if it blows up here, you defined a method with the exact same attribute and priority as one already found

                        // wrap in trace class
                        var comp = mi.Invoke(RebornCoRoutine.Instance, null) as Composite;


                        matchedMethods.Add(attribute, comp);

                        CurrentBehaviorType = 0;
                    }
                }
            }
            // If we found no methods, rofls!
            if (matchedMethods.Count <= 0)
            {
                return null;
            }

            var result = new PrioritySelector();
            foreach (var kvp in matchedMethods.OrderByDescending(mm => mm.Key.PriorityLevel))
            {
                result.AddChild(kvp.Value);
                behaviourCount++;
            }

            return result;
        }
        /*
        public static void InvokeInitializers(ClassJobType job, GameContext context, bool silent = false)
        {
            BehaviorType behavior = BehaviorType.Initialize;

            if (context == GameContext.None)
            {
                return;
            }

            SilentBehaviorCreation = silent;

            // only load methods once
            if (_methods.Count <= 0)
            {
                // Logger.WriteDebug("Singular Behaviors: building method list");
                foreach (var type in Assembly.GetExecutingAssembly().GetTypes())
                {
                    // All behavior methods should not be generic, and should have zero parameters, with their return types being of type Composite.
                    _methods.AddRange(
                        type.GetMethods(BindingFlags.Static | BindingFlags.Public).Where(
                            mi => !mi.IsGenericMethod && mi.GetParameters().Length == 0).Where(
                                mi => mi.ReturnType.IsAssignableFrom(typeof(Composite))));
                }
                Logger.WriteFile("RebornCo Behaviors: Added " + _methods.Count + " behaviors");
            }

            // find all initialization methods for this class/spec/context
            var matchedMethods = new Dictionary<BehaviorAttribute, MethodInfo>();
            foreach (MethodInfo mi in _methods)
            {
                // If the behavior is set as ignore. Don't use it? Duh?
                if (mi.GetCustomAttributes(typeof(IgnoreBehaviorCountAttribute), false).Any())
                    continue;

                // If there's no behavior attrib, then move along.
                foreach (var a in mi.GetCustomAttributes(typeof(BehaviorAttribute), false))
                {
                    var attribute = a as BehaviorAttribute;
                    if (attribute == null)
                        continue;

                    // Check if our Initialization behavior matches. If not, don't add it!
                    if (IsMatchingMethod(attribute, behavior, context, job))
                    {
                        matchedMethods.Add(attribute, mi);
                    }
                }
            }
        }
        */
        private static bool IsMatchingMethod(BehaviorAttribute attribute, BehaviorType behavior, GameContext context, ClassJobType job)
        {
            if ((attribute.Type & behavior) == 0)
                return false;
            if ((attribute.SpecificContext & context) == 0)
                return false;
            if (attribute.Job != job && attribute.Job != ClassJobType.Adventurer)
                return false;

            Logger.WriteDebug("IsMatchingMethod({0}, {1}, {2}) - {3}, {4}, {5}, {6}", job, behavior,
                context, attribute.Job, attribute.Type, attribute.SpecificContext,
                attribute.PriorityLevel);  
            return true;

        }
    }
}