﻿//////////////////////////////////////////////////
//                 Lancer.cs                    //
//         Part of RebornCo by kaihaider        //
//////////////////////////////////////////////////
using System.Threading.Tasks;
using ff14bot;
using ff14bot.Enums;
using RebornCo.Helpers;
using RebornCo.Settings;
using TreeSharp;

namespace RebornCo.Rotations
{
    public class Lancer : RebornCoRoutine
    {
        public override void OnInitialize()
        {
            WantHealing = false;
            WindowSettings = new LancerSetting();
        }
        public static LancerSetting LancerSettings
        {
            get { return WindowSettings as LancerSetting; }
        }
        public sealed override ClassJobType[] Class
        {
            get
            {
                return new [] 
                { 
                    ClassJobType.Lancer
                };
            }
        }

        [Behavior(BehaviorType.PreCombatBuffs, ClassJobType.Lancer)]
        public Composite LancerCreateBasicPreCombatBuffs()
        {
            return new ActionRunCoroutine(ctx => LancerBuildPreCombatBuffs());
        }
        public static async Task<bool> LancerBuildPreCombatBuffs()
        {
            if (!IsValid()) return false;
            if (Resting)
            {
                if (!Core.Player.IsCasting && await Spell.Cast("Cure", Core.Player, Core.Player.CurrentHealthPercent < 70 && Core.Player.CurrentHealthPercent < Core.Player.CurrentManaPercent)) return true;
                if (!Core.Player.IsCasting && await Spell.Cast("Physick", Core.Player, Core.Player.CurrentHealthPercent < 70 && Core.Player.CurrentHealthPercent < Core.Player.CurrentManaPercent)) return true;
                return false;
            }
            if (await Spell.BuffParty("Protect")) return true;
            if (await Spell.BuffParty("Stoneskin")) return true;
            return await CombatCompanion();
        }
        [Behavior(BehaviorType.Rest, ClassJobType.Lancer)]
        public Composite LancerCreateBasicRest()
        {
            return new ActionRunCoroutine(ctx => DefaultRestBehavior(Core.Player.CurrentTPPercent));
        }
        [Behavior(BehaviorType.CombatBuffs,  ClassJobType.Lancer)]
        public Composite LancerCreateBasicCombatBuffs()
        {
            return new ActionRunCoroutine(ctx => LancerBuildCombatBuffs()); 
        }
        public static async Task<bool> LancerBuildCombatBuffs()
        {
            ShouldPulse = true;
            if (await Spell.Cast("Cure", Core.Player, Core.Player.CurrentHealthPercent < 40)) return true;
            if (await Spell.Cast("Physick", Core.Player, Core.Player.CurrentHealthPercent < 40)) return true;
            if (Spell.GcdTime < LancerSettings.NoneGcdWindowEnd) return false;
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Life Surge", Core.Player, Spell.LastCombo == Combo.VorpalThrust))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Mercy Stroke", Core.Player.CurrentTarget, Core.Player.CurrentTarget != null && Core.Player.CurrentTarget.CurrentHealthPercent < 20))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Leg Sweep", Core.Player.CurrentTarget, LancerSettings.UseLegSweep))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Invigorate", Core.Player, Core.Player.CurrentTPPercent < 50))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Foresight", Core.Player, !Core.Player.HasAura("Foresight")))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Eye for an Eye", Core.Player, true))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Blood for Blood", Core.Player, true))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Quelling Strikes", Core.Player, true))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Hawk's Eye", Core.Player))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Bloodbath", Core.Player, true))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Raging Strikes", Core.Player, true))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Internal Release", Core.Player, true))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Keen Flurry", Core.Player, Core.Player.CurrentHealthPercent < 80))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Foresight", Core.Player, Core.Player.CurrentHealthPercent < 60))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Convalescence", Core.Player, Core.Player.CurrentHealthPercent < 70))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Second Wind", Core.Player, Core.Player.CurrentHealthPercent < 40))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Awareness", Core.Player, Core.Player.CurrentHealthPercent < 30))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Featherfoot", Core.Player, Core.Player.Distance2D() < 5))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            return false;
        }

        [Behavior(BehaviorType.Pull, ClassJobType.Lancer)]
        public Composite LancerCreateBasicPull()
        {
            return new ActionRunCoroutine(ctx => LancerBuildPullBehavior());
        }
        public static async Task<bool> LancerBuildPullBehavior()
        {
            if (Core.Player.CurrentTarget != null && !Core.Player.CurrentTarget.CanAttack || !IsValid()) return false;
            ShouldPulse = true;
            await Target.EnsureValidTarget();
            if (await Movement.PullMove(Core.Player.CurrentTarget)) return true;
            if (LancerSettings.WaitEnergyPull && Core.Player.CurrentTPPercent < LancerSettings.RestEnergy)
            {
                ff14bot.Helpers.Logging.Write("[RebornCo] Waiting for energy before pull!");
                return true;
            }
            if (!Spell.Gcd && await Spell.PullCast("Vorpal Thrust", Core.Player.CurrentTarget, Spell.LastCombo == Combo.TrueThrust && Core.Player.ClassLevel > 3)) return true;
            if (!Spell.Gcd && await Spell.PullCast("Full Thrust", Core.Player.CurrentTarget, Spell.LastCombo == Combo.VorpalThrust && Core.Player.ClassLevel > 25)) return true;
            if (!Spell.Gcd && await Spell.PullCast("Disembowel", Core.Player.CurrentTarget, Spell.LastCombo == Combo.ImpulseDrive && Core.Player.ClassLevel > 37)) return true;
            if (!Spell.Gcd && await Spell.PullCast("Chaos Thrust", Core.Player.CurrentTarget, Spell.LastCombo == Combo.Disembowel && Core.Player.ClassLevel > 37)) return true;
            if (!Spell.Gcd && await Spell.PullCast("Impulse Drive", Core.Player.CurrentTarget)) return true;
            return await Spell.PullCast("True Thrust", Core.Player.CurrentTarget);
        }

        [Behavior(BehaviorType.Combat, ClassJobType.Lancer)]
        public Composite LancerCreateBasicCombat()
        {
            return new ActionRunCoroutine(ctx => LancerBuildBasicCombat());
        }
        public static async Task<bool> LancerBuildBasicCombat()
        {
            await Target.EnsureValidTarget();
            await Movement.MoveToTarget();
            if (Core.Player.CurrentTarget != null && ff14bot.Managers.MovementManager.IsMoving
                && !(Core.Player.CurrentTarget.InLineOfSight() && Core.Player.CurrentTarget.Distance2D() < Movement.ObjectMeleeRange(Core.Player.CurrentTarget))) return true;
            if (!Spell.Gcd && Spell.LastCombo == Combo.ImpulseDrive
                && Core.Player.ClassLevel > 37)
            {
                return await Spell.Cast("Disembowel", Core.Player.CurrentTarget);
            }
            if (!Spell.Gcd && Spell.LastCombo == Combo.Disembowel
                && Core.Player.ClassLevel == 50)
            {
                return await Spell.Cast("Chaos Thrust", Core.Player.CurrentTarget);
            }
            if (!Spell.Gcd && Spell.LastCombo == Combo.TrueThrust
                && Core.Player.ClassLevel > 3)
            {
                return await Spell.Cast("Vorpal Thrust", Core.Player.CurrentTarget);
            }
            if (!Spell.Gcd && Spell.LastCombo == Combo.VorpalThrust
                && Core.Player.ClassLevel > 25)
            {
                return await Spell.Cast("Full Thrust", Core.Player.CurrentTarget);
            }
            if (!Spell.Gcd && await Spell.CastLock("Heavy Thrust", Core.Player.CurrentTarget,!Core.Player.HasAura("Heavy Thrust",false,LancerSettings.HeavyThrustClip))) return true;
            if (!Spell.Gcd && await Spell.CastLock("Impulse Drive", Core.Player.CurrentTarget, Core.Player.CurrentTarget != null 
                && Core.Player.ClassLevel > 37  && !Core.Player.CurrentTarget.HasAura("Disembowel", false, LancerSettings.DisembowelClip))) return true;
            if (!Spell.Gcd && await Spell.CastLock("Phlebotomize", Core.Player.CurrentTarget, !Core.Player.CurrentTarget.HasAura("Phlebotomize", true, LancerSettings.PhlebotomizeClip))) return true;
            if (!Spell.Gcd && await Spell.Cast("Piercing Talon", Core.Player.CurrentTarget, Core.Player.CurrentTarget != null && Core.Player.CurrentTarget.Distance2D() < 15 && Core.Player.CurrentTarget.Distance2D() > (Movement.MeleeRange + LancerSettings.DistanceFromMeleeRangeToUseRanged))) return true;
            if (!Spell.Gcd && await Spell.CastLock("Virus", Core.Player.CurrentTarget, Core.Player.CurrentTarget != null && !Core.Player.CurrentTarget.HasAura("Virus"))) return true; 
            if (!Spell.Gcd && await Spell.CastLock("Venomous Bite", Core.Player.CurrentTarget, !Core.Player.CurrentTarget.HasAura("Venomous Bite", true))) return true;
            if (!Spell.Gcd && await Spell.CastLock("Fracture", Core.Player.CurrentTarget, Core.Player.CurrentTarget != null && !Core.Player.CurrentTarget.HasAura("Fracture"))) return true;
            if (!Spell.Gcd && await Spell.Cast("True Thrust", Core.Player.CurrentTarget)) return true;
            return false;

        }
    }
}