﻿
using System;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Media;
using Buddy.Coroutines;
using ff14bot;
using ff14bot.Enums;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.Objects;
using RebornCo.Helpers;
using RebornCo.Settings;
using TreeSharp;

namespace RebornCo.Rotations
{
    public class Ninja : RebornCoRoutine
    {
        public override void OnInitialize()
        {
            WantHealing = false;
            WindowSettings = new RogueSetting();
        }
        public sealed override ClassJobType[] Class
        {
            get
            {
                return new[] 
                { ClassJobType.Rogue, ClassJobType.Ninja};
            }
        }
        public static RogueSetting RogueSettings
        {
            get { return WindowSettings as RogueSetting; }
        }
        public static bool MudraReady;

        [Behavior(BehaviorType.PreCombatBuffs, ClassJobType.Ninja, GameContext.All, 1)]
        public Composite NinjaCreateBasicPreCombatBuffs()
        {
            return new ActionRunCoroutine(ctx => NinjaBuildPreCombatBuffs());
        }
        public static async Task<bool> NinjaBuildPreCombatBuffs()
        {
            if (Core.Player.CurrentTarget == null || !IsValid()) return false;

            if (Resting)
            {
                if (!Core.Player.IsCasting && await Spell.Cast("Cure", Core.Player, Core.Player.CurrentHealthPercent < 70 && Core.Player.CurrentHealthPercent < Core.Player.CurrentManaPercent)) return true;
                if (!Core.Player.IsCasting && await Spell.Cast("Physick", Core.Player, Core.Player.CurrentHealthPercent < 70 && Core.Player.CurrentHealthPercent < Core.Player.CurrentManaPercent)) return true;
                return false;
            }

            if (MudraReady && DateTime.UtcNow.TimeOfDay > MudraTimeReady)
            {
                if (!Core.Player.HasAura("Mudra") && (NinjutsuInCooldown() || NinjutsuCanMudra()))
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] No Mudra?");
                    MudraReady = false;
                }
                if (Core.Player.CurrentTarget.InLineOfSight() && Core.Player.Distance2D(Core.Player.CurrentTarget.Location) < 15
                    && await CastNinjutsu(Core.Player.CurrentTarget))
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] Casting Ninjutsu");
                }
                return true;
            }
            if (MudraReady) { return false; } 
            if (!Spell.Gcd && Core.Player.CurrentTarget != null && Core.Player.CurrentTarget.CanAttack &&
                !Core.Player.HasAura("Huton") && NinjutsuCanMudra() && Core.Player.ClassLevel > 44
                && Core.Player.CurrentTarget.InLineOfSight() && Core.Player.Distance2D(Core.Player.CurrentTarget.Location) < 15)
            {
                await Coroutine.Sleep(150);
                Logging.Write(Colors.Orchid, "[RebornCo] Preparing Huton");
                if (await Spell.Cast("Jin", Core.Player, true, false, false, true) || RogueSettings.IgnoreMudraDoAction)
                {
                    MudraTimeReady = DateTime.UtcNow.TimeOfDay + TimeSpan.FromMilliseconds(510);
                }
                else
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] Mudra Failed!");
                    return false;
                }
                MudraReady = true;
                if (RogueSettings.MoveStopNinjutsu) Movement.StopMove();
                else await Movement.MoveToTarget();
                await Coroutine.Wait(500, () => DateTime.UtcNow.TimeOfDay > MudraTimeReady);
                if (await Spell.Cast("Chi", Core.Player, true, false, false, true) || RogueSettings.IgnoreMudraDoAction)
                {
                    MudraTimeReady = DateTime.UtcNow.TimeOfDay + TimeSpan.FromMilliseconds(510);
                }
                else
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] Mudra Failed!");
                    return false;
                }
                if (!RogueSettings.MoveStopNinjutsu) await Movement.MoveToTarget();
                await Coroutine.Wait(500, () => DateTime.UtcNow.TimeOfDay > MudraTimeReady);
                if (await Spell.Cast("Ten", Core.Player, true, false, false, true) || RogueSettings.IgnoreMudraDoAction)
                {
                    MudraTimeReady = DateTime.UtcNow.TimeOfDay + TimeSpan.FromMilliseconds(510);
                }
                else
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] Mudra Failed!");
                    return false;
                }
                return true;
            }
            return MudraReady;
        }
        [Behavior(BehaviorType.PreCombatBuffs)]
        public Composite RogueCreateBasicPreCombatBuffs()
        {
            return new ActionRunCoroutine(ctx => RogueBuildPreCombatBuffs());
        }
        public static async Task<bool> RogueBuildPreCombatBuffs()
        {
            if (Core.Player.CurrentTarget == null || !IsValid()) return false;


            if (await Spell.BuffParty("Protect")) return true;
            if (await Spell.BuffParty("Stoneskin")) return true;
            if (await Spell.Cast("Kiss of the Viper", Core.Player, !(Core.Player.HasAura("Kiss of the Viper")))) return true;
            if (await Spell.Cast("Kiss of the Wasp", Core.Player, !(Core.Player.HasAura("Kiss of the Viper") || Core.Player.HasAura("Kiss of the Wasp")))) return true;

            if (Resting)
            {
                if (!Core.Player.IsCasting && await Spell.Cast("Cure", Core.Player, Core.Player.CurrentHealthPercent < 70 && Core.Player.CurrentHealthPercent < Core.Player.CurrentManaPercent)) return true;
                if (!Core.Player.IsCasting && await Spell.Cast("Physick", Core.Player, Core.Player.CurrentHealthPercent < 70 && Core.Player.CurrentHealthPercent < Core.Player.CurrentManaPercent)) return true;
                return false;
            }
            return await CombatCompanion();
        }

        [Behavior(BehaviorType.Rest)]
        public Composite NinjaCreateBasicRest()
        {
            return new PrioritySelector( 
                new Decorator(ctx =>  Core.Player.HasAura("Hidden") && Resting, 
                    new ActionRunCoroutine(ctx => Spell.Cast("Hide"))),
                new ActionRunCoroutine(ctx => DefaultRestBehavior(Core.Player.CurrentTPPercent))); 
        }
        
        [Behavior(BehaviorType.CombatBuffs, ClassJobType.Adventurer, GameContext.All, 2)]
        public Composite TopCreateBasicCombatBuffs()
        {
            return new ActionRunCoroutine(ctx => TopBuildCombatBuffs());
        }

        public static async Task<bool> TopBuildCombatBuffs()
        {
            ShouldPulse = true;
            await Target.EnsureValidTarget();
            await Movement.MoveToTarget();
            return false;
        }

        [Behavior(BehaviorType.CombatBuffs, ClassJobType.Ninja, GameContext.All, 1)]
        public Composite NinjaCreateBasicCombatBuffs()
        {
            return new ActionRunCoroutine(ctx => NinjaBuildCombatBuffs());
        }
        public static async Task<bool> NinjaBuildCombatBuffs()
        {
            if (MudraReady && Core.Player.CurrentTarget != null && DateTime.UtcNow.TimeOfDay > MudraTimeReady)
            {
                if (!Core.Player.HasAura("Mudra") && (NinjutsuInCooldown() || NinjutsuCanMudra()))
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] No Mudra?");
                    MudraReady = false;
                }
                if (Core.Player.CurrentTarget.InLineOfSight() && Core.Player.Distance2D(Core.Player.CurrentTarget.Location) < 15
                    && await CastNinjutsu(Core.Player.CurrentTarget))
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] Casting Ninjutsu");
                }
                return true;
            }
            if (MudraReady) { return true; }
            if (!Spell.UsedNoneGcd && Spell.GcdTime < RogueSettings.NinjutsuWindowStart && Spell.GcdTime > RogueSettings.NoneGcdWindowEnd && Core.Player.ClassLevel > 44 && NinjutsuCanMudra()
                && !Core.Player.HasAura("Huton", false, RogueSettings.HutonClip))
            {
                Spell.UsedNoneGcd = true;
                await Coroutine.Sleep(150);
                Logging.Write(Colors.Orchid, "[RebornCo] Preparing Huton");
                if (await Spell.Cast("Jin", Core.Player, true, false, false, true) || RogueSettings.IgnoreMudraDoAction)
                {
                    MudraTimeReady = DateTime.UtcNow.TimeOfDay + TimeSpan.FromMilliseconds(510);
                }
                else
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] Mudra Failed!");
                    return true;
                }
                MudraReady = true;
                if (RogueSettings.MoveStopNinjutsu) Movement.StopMove();
                else await Movement.MoveToTarget();
                await Coroutine.Wait(500, () => DateTime.UtcNow.TimeOfDay > MudraTimeReady);
                if (await Spell.Cast("Chi", Core.Player, true, false, false, true) || RogueSettings.IgnoreMudraDoAction)
                {
                    MudraTimeReady = DateTime.UtcNow.TimeOfDay + TimeSpan.FromMilliseconds(510);
                }
                else
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] Mudra Failed!");
                    return true;
                }
                if (!RogueSettings.MoveStopNinjutsu) await Movement.MoveToTarget();
                await Coroutine.Wait(500, () => DateTime.UtcNow.TimeOfDay > MudraTimeReady);
                if (await Spell.Cast("Ten", Core.Player, true, false, false, true) || RogueSettings.IgnoreMudraDoAction)
                {
                    MudraTimeReady = DateTime.UtcNow.TimeOfDay + TimeSpan.FromMilliseconds(510);
                }
                else
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] Mudra Failed!");
                    return true;
                }
                return true;
            }

            if (!Spell.UsedNoneGcd && Spell.GcdTime < RogueSettings.NinjutsuWindowStart && Spell.GcdTime > RogueSettings.NoneGcdWindowEnd
                && Core.Player.ClassLevel == 50 && Core.Player.CurrentTarget != null && Core.Player.CurrentTarget.IsBehind && NinjutsuCanMudra()
                    && Core.Player.Distance2D(Core.Player.CurrentTarget.Location) < 15
                    && CanTrick())
            {
                Spell.UsedNoneGcd = true;
                await Coroutine.Sleep(150);
                Logging.Write(Colors.Orchid, "[RebornCo] Preparing Suiton");
                if (await Spell.Cast("Ten", Core.Player, true, false, false, true) || RogueSettings.IgnoreMudraDoAction)
                {
                    MudraTimeReady = DateTime.UtcNow.TimeOfDay + TimeSpan.FromMilliseconds(510);
                }
                else
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] Mudra Failed!");
                    return true;
                }
                MudraReady = true;
                if (RogueSettings.MoveStopNinjutsu) Movement.StopMove();
                else await Movement.MoveToTarget();
                await Coroutine.Wait(500, () => DateTime.UtcNow.TimeOfDay > MudraTimeReady);
                if (await Spell.Cast("Chi", Core.Player, true, false, false, true) || RogueSettings.IgnoreMudraDoAction)
                {
                    MudraTimeReady = DateTime.UtcNow.TimeOfDay + TimeSpan.FromMilliseconds(510);
                }
                else
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] Mudra Failed!");
                    return true;
                }
                if (!RogueSettings.MoveStopNinjutsu) await Movement.MoveToTarget();
                await Coroutine.Wait(500, () => DateTime.UtcNow.TimeOfDay > MudraTimeReady);
                if (await Spell.Cast("Jin", Core.Player, true, false, false, true) || RogueSettings.IgnoreMudraDoAction)
                {
                    MudraTimeReady = DateTime.UtcNow.TimeOfDay + TimeSpan.FromMilliseconds(510);
                }
                else
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] Mudra Failed!");
                    return true;
                }
                return true;
            }
            if (!Spell.UsedNoneGcd && Spell.GcdTime < RogueSettings.NinjutsuWindowStart && Spell.GcdTime > RogueSettings.NoneGcdWindowEnd && Core.Player.CurrentTarget != null && RogueSettings.Aoe && Core.Player.CurrentTarget.CurrentHealthPercent > RogueSettings.MinNinjutsu
                && Core.Player.ClassLevel > 44 && NinjutsuCanMudra()
                && Target.EnemiesNearTarget(5, Core.Player) > RogueSettings.AoeCount
                     && Movement.IsInSafeMeleeRange(Core.Player.CurrentTarget)
                     && Core.Player.CurrentTarget.InLineOfSight() && !Core.Player.CurrentTarget.HasAura("Doton"))
            {
                Spell.UsedNoneGcd = true;
                await Coroutine.Sleep(150);
                Logging.Write(Colors.Orchid, "[RebornCo] Preparing Doton");
                if (await Spell.Cast("Ten", Core.Player, true, false, false, true) || RogueSettings.IgnoreMudraDoAction)
                {
                    MudraTimeReady = DateTime.UtcNow.TimeOfDay + TimeSpan.FromMilliseconds(510);
                }
                else
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] Mudra Failed!");
                    return true;
                }
                MudraReady = true;
                if (RogueSettings.MoveStopNinjutsu) Movement.StopMove();
                else await Movement.MoveToTarget();
                await Coroutine.Wait(500, () => DateTime.UtcNow.TimeOfDay > MudraTimeReady);
                if (await Spell.Cast("Jin", Core.Player, true, false, false, true) || RogueSettings.IgnoreMudraDoAction)
                {
                    MudraTimeReady = DateTime.UtcNow.TimeOfDay + TimeSpan.FromMilliseconds(510);
                }
                else
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] Mudra Failed!");
                    return true;
                }
                if (!RogueSettings.MoveStopNinjutsu) await Movement.MoveToTarget();
                await Coroutine.Wait(500, () => DateTime.UtcNow.TimeOfDay > MudraTimeReady);
                if (await Spell.Cast("Chi", Core.Player, true, false, false, true) || RogueSettings.IgnoreMudraDoAction)
                {
                    MudraTimeReady = DateTime.UtcNow.TimeOfDay + TimeSpan.FromMilliseconds(510);
                }
                else
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] Mudra Failed!");
                    return true;
                }
                return true;
            }
            if (!Spell.UsedNoneGcd && Spell.GcdTime < RogueSettings.NinjutsuWindowStart && Spell.GcdTime > RogueSettings.NoneGcdWindowEnd && Core.Player.ClassLevel > 34 && Core.Player.CurrentTarget != null
                && RogueSettings.Aoe && Core.Player.CurrentTarget.CurrentHealthPercent > RogueSettings.MinNinjutsu
                && Core.Player.CurrentTarget != null && Core.Player.CurrentTarget.InLineOfSight()
              && NinjutsuCanMudra() && Target.EnemiesNearTarget(5) > RogueSettings.AoeCount
                     && Core.Player.Distance2D(Core.Player.CurrentTarget.Location) < 15)
            {
                Spell.UsedNoneGcd = true;
                await Coroutine.Sleep(150);
                Logging.Write(Colors.Orchid, "[RebornCo] Preparing Katon");
                if (await Spell.Cast("Chi", Core.Player, true, false, false, true) || RogueSettings.IgnoreMudraDoAction)
                {
                    MudraTimeReady = DateTime.UtcNow.TimeOfDay + TimeSpan.FromMilliseconds(510);
                }
                else
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] Mudra Failed!");
                    return true;
                }
                MudraReady = true;
                if (RogueSettings.MoveStopNinjutsu) Movement.StopMove();
                else await Movement.MoveToTarget();
                await Coroutine.Wait(500, () => DateTime.UtcNow.TimeOfDay > MudraTimeReady);
                if (await Spell.Cast("Ten", Core.Player, true, false, false, true) || RogueSettings.IgnoreMudraDoAction)
                {
                    MudraTimeReady = DateTime.UtcNow.TimeOfDay + TimeSpan.FromMilliseconds(510);
                }
                else
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] Mudra Failed!");
                    return true;
                }
                return true;
            }
            if (!Spell.UsedNoneGcd && Spell.GcdTime < RogueSettings.NinjutsuWindowStart && Spell.GcdTime > RogueSettings.NoneGcdWindowEnd && Core.Player.CurrentTarget != null && Core.Player.ClassLevel > 29
                && Core.Player.CurrentTarget.CurrentHealthPercent > RogueSettings.MinNinjutsu
                && Core.Player.CurrentTarget.InLineOfSight() &&
                     Core.Player.Distance2D(Core.Player.CurrentTarget.Location) < 15 &&
                     NinjutsuCanMudra())
            {
                Spell.UsedNoneGcd = true;
                await Coroutine.Sleep(150);
                Logging.Write(Colors.Orchid, "[RebornCo] Preparing Raiton");
                if (await Spell.Cast("Ten", Core.Player, true, false, false, true) || RogueSettings.IgnoreMudraDoAction)
                {
                    MudraTimeReady = DateTime.UtcNow.TimeOfDay + TimeSpan.FromMilliseconds(510);
                }
                else
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] Mudra Failed!");
                    return true;
                }
                MudraReady = true;
                if (RogueSettings.MoveStopNinjutsu) Movement.StopMove();
                else await Movement.MoveToTarget();
                await Coroutine.Wait(500, () => DateTime.UtcNow.TimeOfDay > MudraTimeReady);
                if (await Spell.Cast("Chi", Core.Player, true, false, false, true) || RogueSettings.IgnoreMudraDoAction)
                {
                    MudraTimeReady = DateTime.UtcNow.TimeOfDay + TimeSpan.FromMilliseconds(510);
                }
                else
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] Mudra Failed!");
                    return true;
                }
                return true;
            }
            return MudraReady;
        }

        [Behavior(BehaviorType.CombatBuffs)]
        public Composite RogueCreateBasicCombatBuffs()
        {
            return new ActionRunCoroutine(ctx => RogueBuildCombatBuffs()); 
        }
        public static async Task<bool> RogueBuildCombatBuffs()
        {
            if (await Spell.Cast("Cure", Core.Player, Core.Player.CurrentHealthPercent < 40)) return true;
            if (await Spell.Cast("Physick", Core.Player, Core.Player.CurrentHealthPercent < 40)) return true;

            if (Spell.GcdTime < RogueSettings.NoneGcdWindowEnd) return false;

            if (!Spell.UsedNoneGcd && await Spell.CastLock("Kassatsu", Core.Player, ShouldKassatsu))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Perfect Dodge", Core.Player, RogueSettings.CooldownsUsable && Core.Player.CurrentHealthPercent < 40))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Invigorate", Core.Player, Core.Player.CurrentTPPercent < 50))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Blood for Blood", Core.Player, true))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Internal Release", Core.Player, true))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Foresight", Core.Player, Core.Player.CurrentHealthPercent < 40))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Eye for an Eye", Core.Player, true))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Quelling Strikes", Core.Player, true))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Hawk's Eye", Core.Player))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Bloodbath", Core.Player, true))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Raging Strikes", Core.Player, true))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Keen Flurry", Core.Player, Core.Player.CurrentHealthPercent < 80))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Foresight", Core.Player, Core.Player.CurrentHealthPercent < 60))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Convalescence", Core.Player, Core.Player.CurrentHealthPercent < 70))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Second Wind", Core.Player, Core.Player.CurrentHealthPercent < 40))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Awareness", Core.Player, Core.Player.CurrentHealthPercent < 30))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Featherfoot", Core.Player, Core.Player.Distance2D() < 5))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            return false;
        }

        [Behavior(BehaviorType.Pull)]
        public Composite NinjaCreateBasicPull()
        {
            return new ActionRunCoroutine(ctx => NinjaBuildPullBehavior()); 
        }
        public static async Task<bool> NinjaBuildPullBehavior()
        {
            if (Core.Player.CurrentTarget != null && !Core.Player.CurrentTarget.CanAttack || !IsValid()) return false;
            await Target.EnsureValidTarget();
            ShouldPulse = true;
            if (await Movement.PullMove(Core.Player.CurrentTarget)) return true;
            if (RogueSettings.WaitEnergyPull && Core.Player.CurrentTPPercent < RogueSettings.RestEnergy)
            {
                Logging.Write("[RebornCo] Waiting for energy before pull!");
                return true;
            }
            if (MudraReady||Core.Player.HasAura("Mudra"))  await Movement.MoveToTarget();
            if (MudraReady && DateTime.UtcNow.TimeOfDay > MudraTimeReady)
            {
                if (!Core.Player.HasAura("Mudra") && (NinjutsuInCooldown() || NinjutsuCanMudra()))
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] No Mudra?");
                    MudraReady = false;
                }
                if (Core.Player.CurrentTarget.InLineOfSight() && Core.Player.Distance2D(Core.Player.CurrentTarget.Location) < 15
                    && await CastNinjutsu(Core.Player.CurrentTarget))
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] Casting Ninjutsu");
                }
                return true;
            }
            if (MudraReady) return false;
            
            if (await Spell.Cast("Hide", Core.Player, RogueSettings.UseHide && (Core.Player.ClassLevel > 21 && Core.Player.CurrentTarget!=null && !Core.Player.CurrentTarget.IsBehind && !Core.Player.CurrentTarget.IsFlanking || Core.Player.ClassLevel == 50)
                && !Core.Player.HasAura("Hidden"))) return true;
            if (!Spell.Gcd && await Spell.PullCast("Sneak Attack", Core.Player.CurrentTarget, (RogueSettings.CooldownsUsable || RogueSettings.UseHide) && Core.Player.HasAura("Hidden") && Core.Player.CurrentTarget != null && !Core.Player.CurrentTarget.IsFlanking && !Core.Player.CurrentTarget.IsBehind)) return true;
            if (!Spell.Gcd && await Spell.PullCast("Trick Attack", Core.Player.CurrentTarget, (RogueSettings.CooldownsUsable || RogueSettings.UseHide) && Core.Player.HasAura("Hidden") && Core.Player.CurrentTarget != null && Core.Player.CurrentTarget.IsBehind)) return true;
            if (!Spell.Gcd && await Spell.PullCast("Shadow Fang", Core.Player.CurrentTarget, Spell.LastCombo == Combo.SpinningEdge && (Spell.CachedCombo == Combo.Flushed|| Spell.CachedCombo == Combo.ShadowFang)
                && !Core.Player.CurrentTarget.HasAura("Shadow Fang", true))) return true;
            if (!Spell.Gcd && await Spell.PullCast("Gust Slash", Core.Player.CurrentTarget, Spell.LastCombo == Combo.SpinningEdge&& (Spell.CachedCombo == Combo.Flushed|| Spell.CachedCombo == Combo.GustSlash))) return true;
            if (!Spell.Gcd && await Spell.PullCast("Dancing Edge", Core.Player.CurrentTarget, Spell.LastCombo == Combo.GustSlash && (Spell.CachedCombo == Combo.Flushed|| Spell.CachedCombo == Combo.DancingEdge)
                && Core.Player.CurrentTarget != null && (Core.Player.CurrentTarget.IsFlanking || !(Core.Player.CurrentTarget.HasAura("Dancing Edge") || Core.Player.CurrentTarget.HasAura("Storm's Eye"))))) return true;
            if (!Spell.Gcd && await Spell.PullCast("Aeolian Edge", Core.Player.CurrentTarget, Spell.LastCombo == Combo.GustSlash&& (Spell.CachedCombo == Combo.Flushed|| Spell.CachedCombo == Combo.AeolianEdge))) return true;
            if (!Spell.Gcd && await Spell.PullCast("Mutilate", Core.Player.CurrentTarget, Core.Player.CurrentTarget != null && Core.Player.CurrentTarget.CurrentHealthPercent > RogueSettings.MinDotHp
                && !Core.Player.CurrentTarget.HasAura("Mutilation", true, RogueSettings.MutilationClip))) return true;
            if (await Spell.PullCast("Spinning Edge", Core.Player.CurrentTarget)) return true;
            return false;
        }

        [Behavior(BehaviorType.Combat)]
        public Composite NinjaCreateBasicCombat()
        {
            return new ActionRunCoroutine(ctx => NinjaBuildBasicCombat()); 
        }
        public static async Task<bool> NinjaBuildBasicCombat()
        {
            if (Core.Player.IsDead || Core.Player.CurrentTarget == null) return false;

            if (Core.Player.CurrentTarget != null && MovementManager.IsMoving
                && !(Core.Player.CurrentTarget.InLineOfSight() && Core.Player.CurrentTarget.Distance2D() < Movement.ObjectMeleeRange(Core.Player.CurrentTarget))) return true;
            if (await Spell.Cast("Sneak Attack", Core.Player.CurrentTarget, RogueSettings.SneakAttack && RogueSettings.CooldownsUsable && Core.Player.ClassLevel > 21
                && Core.Player.HasAura("Hidden") && Core.Player.CurrentTarget != null && !Core.Player.CurrentTarget.IsFlanking && !Core.Player.CurrentTarget.IsBehind)) return true;
            if (await Spell.Cast("Trick Attack", Core.Player.CurrentTarget, RogueSettings.CooldownsUsable && Core.Player.ClassLevel == 50 && (Core.Player.HasAura("Hidden") || Core.Player.HasAura("Suiton")) && Core.Player.CurrentTarget != null && Core.Player.CurrentTarget.IsBehind)) return true;
            if (!Spell.Gcd && Spell.LastCombo == Combo.SpinningEdge && Core.Player.CurrentTarget != null) 
            {
                if (await Spell.CastLock("Shadow Fang", Core.Player.CurrentTarget, Core.Player.ClassLevel > 45
                    && (Spell.CachedCombo == Combo.ShadowFang || Spell.CachedCombo == Combo.Flushed
                    && Core.Player.CurrentTarget.CurrentHealthPercent > RogueSettings.MinDotHp
                    && !Core.Player.CurrentTarget.HasAura("Shadow Fang", true, RogueSettings.ShadowFangClip) && (Core.Player.CurrentTarget.HasAura("Dancing Edge", false, RogueSettings.DancingEdgeClip)
                  || Core.Player.CurrentTarget.HasAura("Storm's Eye", false, RogueSettings.DancingEdgeClip))
                    ))) return true;
                return await Spell.Cast("Gust Slash", Core.Player.CurrentTarget, (Spell.CachedCombo == Combo.Flushed|| Spell.CachedCombo == Combo.GustSlash));
            }
            if (!Spell.Gcd && Spell.LastCombo == Combo.GustSlash && Core.Player.CurrentTarget != null && Core.Player.ClassLevel > 25)
            {
                if (await Spell.CastLock("Dancing Edge", Core.Player.CurrentTarget, Core.Player.ClassLevel > 37
                  && (Spell.CachedCombo == Combo.DancingEdge || Spell.CachedCombo == Combo.Flushed
                  && Core.Player.CurrentTarget.CurrentHealthPercent > RogueSettings.MinDotHp 
                  && !(Core.Player.CurrentTarget.HasAura("Dancing Edge", false, RogueSettings.DancingEdgeClip) 
                  || Core.Player.CurrentTarget.HasAura("Storm's Eye", false, RogueSettings.DancingEdgeClip))))) return true;
              return await Spell.Cast("Aeolian Edge", Core.Player.CurrentTarget, Core.Player.ClassLevel > 25 
                  && (Spell.CachedCombo == Combo.Flushed || Spell.CachedCombo == Combo.AeolianEdge));
            }
            if (!Spell.Gcd && await Spell.Cast("Spinning Edge", Core.Player.CurrentTarget, Core.Player.CurrentTarget != null && Core.Player.CurrentTarget.CurrentHealthPercent > RogueSettings.MinDotHp
                    && (Core.Player.ClassLevel > 45 && !Core.Player.CurrentTarget.HasAura("Shadow Fang", true, RogueSettings.ShadowFangClip) 
                    || Core.Player.ClassLevel > 37 && !(Core.Player.CurrentTarget.HasAura("Dancing Edge", false, RogueSettings.DancingEdgeClip)
                  || Core.Player.CurrentTarget.HasAura("Storm's Eye", false, RogueSettings.DancingEdgeClip))))) return true;
            if (!Spell.Gcd && await Spell.CastLock("Mutilate", Core.Player.CurrentTarget, Core.Player.CurrentTarget != null && Core.Player.CurrentTarget.CurrentHealthPercent > RogueSettings.MinDotHp
                && !Core.Player.CurrentTarget.HasAura("Mutilation", true, RogueSettings.MutilationClip))) return true;
            if (!Spell.Gcd && await Spell.Cast("Death Blossom", Core.Player.CurrentTarget, RogueSettings.Aoe && Target.mNearbyEnemyUnits() != null && Target.mNearbyEnemyUnits().Count(unit => unit.Distance2D() <= 5) >= RogueSettings.AoeCount)) return true;
            if (!Spell.Gcd && await Spell.Cast("Throwing Dagger", Core.Player.CurrentTarget, Core.Player.CurrentTarget != null && Core.Player.CurrentTarget.Distance2D() > Movement.MeleeRange + 4)) return true;
           // if (!Gcd && await Spell.Cast("Haymaker", Core.Player.CurrentTarget, true, false, false, true)&&Spell.GcdRogue())) return true;
            if (!Spell.Gcd && await Spell.Cast("Virus", Core.Player.CurrentTarget, Core.Player.CurrentTarget != null && !Core.Player.CurrentTarget.HasAura("Virus"))) return true; 
            if (!Spell.Gcd && await Spell.CastLock("Venomous Bite", Core.Player.CurrentTarget, Core.Player.CurrentTarget != null && Core.Player.CurrentTarget.CurrentHealthPercent > RogueSettings.MinDotHp && !Core.Player.CurrentTarget.HasAura("Venomous Bite"))) return true;
            if (!Spell.Gcd && await Spell.CastLock("Fracture", Core.Player.CurrentTarget, Core.Player.CurrentTarget != null && !Core.Player.CurrentTarget.HasAura("Fracture", true))) return true;
            if (!Spell.Gcd && await Spell.Cast("Spinning Edge", Core.Player.CurrentTarget)) return true;
            if (Spell.GcdTime < RogueSettings.NoneGcdWindowEnd) return false;
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Assassinate", Core.Player.CurrentTarget, Core.Player.CurrentTarget != null && Core.Player.CurrentTarget.CurrentHealthPercent < 20))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Mercy Stroke", Core.Player.CurrentTarget, Core.Player.CurrentTarget != null && Core.Player.CurrentTarget.CurrentHealthPercent < 20))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Mug", Core.Player.CurrentTarget))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            if (!Spell.UsedNoneGcd && await Spell.CastLock("Jugulate", Core.Player.CurrentTarget))
            {
                Spell.UsedNoneGcd = true;
                return true;
            }
            return false;
        }

        public static bool ShouldKassatsu;

        public static bool CanTrick()
        {//might want to increase cooldown check
            if (Extensions.DoubleCastPreventionDict.Contains(null, "Trick Attack")) return false;
            SpellData data;
            if (!Actionmanager.CurrentActions.TryGetValue("Trick Attack", out data)) return false;
            var cooldown = data.Cooldown.TotalMilliseconds;
            if (cooldown < 2500 || Math.Abs(data.BaseCooldown.TotalMilliseconds - cooldown) < 50)
            {
                ShouldKassatsu = true;
                return true;
            }
            Extensions.UpdateDoubleCastDictSpecific("Trick Attack", null, (cooldown - 2000 - WindowSettings.SpellQueue)/1000);
            ShouldKassatsu = false;
            return false;
        }

#pragma warning disable 1998
        public static async Task<bool> CastNinjutsu(GameObject o)
#pragma warning restore 1998
        {
            //if (!CanCast(GetSpellData("Ninjutsu"), Core.Player)) return false;
            if (Core.Player == null) return false;
            return !NinjutsuInCooldown() && Actionmanager.DoAction("Ninjutsu", (o ?? Core.Player));
          
        }

        public static bool NinjutsuInCooldown()
        {
            return (Spell.GetSpellData("Ninjutsu").Cooldown.TotalMilliseconds > 600);
        }

        public static bool NinjutsuCanMudra()
        {
            return (Spell.GetSpellData("Ninjutsu").Cooldown.TotalMilliseconds < 1);
        }

        public static TimeSpan MudraTimeReady;
    }
}
